/*
  Hochschule Hamm-Lippstadt

  Christian Schwinne
  TCS34725-Farbsensor Usermod zur Verwendung im Open-Source-Projekt WLED

  Zwei Bibliotheken sind zu den "lib_deps" zu ergänzen:
  https://github.com/adafruit/Adafruit_BusIO.git
  https://github.com/adafruit/Adafruit_TCS34725.git
*/
#pragma once

#include "wled.h"
#include <Wire.h>
#include "Adafruit_TCS34725.h"

class TCS34725Usermod : public Usermod {
  private:
    Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_2_4MS, TCS34725_GAIN_60X);

    unsigned long lastTime = 0;
    uint8_t r, g, b;
    uint16_t r_raw, g_raw, b_raw, c_raw;
    int r_integr, g_integr, b_integr;
    uint16_t r_last, g_last, b_last;
    bool enabled = true;

    // config variables default values
    uint8_t integrationtime = TCS34725_INTEGRATIONTIME_2_4MS;
    uint8_t gain = TCS34725_GAIN_60X;

    static const char _name[];
    static const char _enabled[];

  public:
    void setup() {
      tcs.begin();
    }


    void loop() {
      if (millis() - lastTime > 50) { // poll color sensor every 50 ms

        tcs.getRawData(&r_raw, &g_raw, &b_raw, &c_raw);
        // PT1-Filter
        r_integr += r_raw - r_last;
        r_raw = r_integr * 10; // T = 0.1 -> 1/0.1 = 10
        r_last = r_raw;
        g_integr += g_raw - g_last;
        g_raw = g_integr * 10; // T = 0.1 -> 1/0.1 = 10
        g_last = g_raw;
        b_integr += b_raw - b_last;
        b_raw = b_integr * 10; // T = 0.1 -> 1/0.1 = 10
        b_last = b_raw;

        // Normalisierung auf 0-255
        uint16_t sum = r_raw + g_raw + b_raw;
        if (sum) {
          r = gamma8((float)r_raw / sum * 255.0);
          g = gamma8((float)g_raw / sum * 255.0);
          b = gamma8((float)b_raw / sum * 255.0);
        } else {
          r = g = b = 0;
        }
        // Gamma-Korrektur durch WLED

        lastTime = millis();
      }
    }
    

    void addToJsonState(JsonObject& root)
    {
      JsonObject usermod = root[FPSTR(_name)];
      if (usermod.isNull()) {
        usermod = root.createNestedObject(FPSTR(_name));
      }
      usermod["on"] = enabled;
    }


    void readFromJsonState(JsonObject& root)
    {
      JsonObject usermod = root[FPSTR(_name)];
      if (usermod.isNull()) return;
      if (usermod["on"].is<bool>()) {
        enabled = usermod["on"].as<bool>();
      }
    }


    void addToConfig(JsonObject& root)
    {
      JsonObject top = root.createNestedObject("tcs34725Usermod");
      top[F("gain")] = gain;
      top[F("integrationtime")] = integrationtime;
    }


    bool readFromConfig(JsonObject& root)
    {
      JsonObject top = root["tcs34725Usermod"];

      bool configComplete = !top.isNull();

      configComplete &= getJsonValue(top[F("gain")], gain);
      configComplete &= getJsonValue(top[F("integrationtime")], integrationtime);
      tcs.setIntegrationTime(integrationtime);
      tcs.setGain((tcs34725Gain_t) gain);

      return configComplete;
    }


    /*
     * handleOverlayDraw() is called just before every show() (LED strip update frame) after effects have set the colors.
     */
    void handleOverlayDraw()
    {
      if (!enabled) return;
      Segment& mainseg = strip.getMainSegment();
      if (mainseg.mode != FX_MODE_RAINBOW_CYCLE) return;
      for (uint16_t i = mainseg.start; i < mainseg.stop; i++)
      {
        strip.setPixelColor(i, RGBW32(r,g,b,0));
      }
    }

   
    /*
     * getId() allows you to optionally give your V2 usermod an unique ID (please define it in const.h!).
     * This could be used in the future for the system to determine whether your usermod is installed.
     */
    uint16_t getId()
    {
      return USERMOD_ID_TCS34725;
    }
};

const char TCS34725Usermod::_name[]    PROGMEM = "TCS34725";
const char TCS34725Usermod::_enabled[] PROGMEM = "enabled";